/*
								+-----------------------------------+
								|                                   |
								|   ***  Dude mesh generator  ***   |
								|                                   |
								|   Copyright   -tHE SWINe- 2007   |
								|                                   |
								|              Dude.h               |
								|                                   |
								+-----------------------------------+
*/

#ifndef __DUDE_MESH_INCLUDED
#define __DUDE_MESH_INCLUDED

/*
 *	class CDudeMesh
 *		- dude mesh generator (mesh is built from bezier spline)
 */
class CDudeMesh {
protected:
	int n_vertex_num;
	int n_index_num;
	float *p_vertex;
	unsigned short *p_index;

	float f_dude_scale;
	Vector2f v_shadow_size;

	Vector2f p_shadow_vertex[2];
	Vector2f p_shadow_texcoord[2];
	Vector2f v_shadow_offset;

public:
	/*
	 *	CDudeMesh::CDudeMesh(float f_base_diameter, int n_subsect_size = 5, int n_side_num = 12)
	 *		- default constructor
	 *		- creates a new mesh having n_subsect_size vertices in one spline subsection
	 *		  (there are three subsections, reasonable value is 5) and n_side_num sides
	 *		- f_base_diameter is required base diameter
	 *		- call b_Status() to see wheter constructor succeeded
	 */
	CDudeMesh(float f_base_diameter, int n_subsect_size = 5, int n_side_num = 12);

	/*
	 *	CDudeMesh::~CDudeMesh()
	 *		- destructor
	 */
	~CDudeMesh();

	/*
	 *	bool CDudeMesh::b_Status() const
	 *		- returns true in case mesh was created successfuly and can be used, otherwise false
	 */
	bool b_Status() const;

	/*
	 *	float CDudeMesh::f_Scale() const
	 *		- returns mesh scaling factor (radius of dude base)
	 */
	float f_Scale() const;

	/*
	 *	void CDudeMesh::PrepareShadow(Vector2f v_dude_shadow_offset)
	 *		- prepares shadow vertex and texture coordinates for shadow texture
	 *		  where dude base center is at v_dude_shadow_offset in OpenGL viewport coordinates
	 */
	void PrepareShadow(Vector2f v_dude_shadow_offset);

	/*
	 *	void CDudeMesh::BindVertexArray() const
	 *		- binds dude vertex array
	 *		- required for drawing
	 */
	void BindVertexArray() const;

	/*
	 *	void CDudeMesh::Draw() const
	 *		- draws the dude using current matrices
	 */
	void Draw() const;

	/*
	 *	void CDudeMesh::Draw(Vector2f v_pos) const
	 *		- draws dude at position v_pos
	 *		- v_org is origin of first stand, v_offset is offset between stands
	 */
	void Draw(Vector2f v_pos) const;

	/*
	 *	void CDudeMesh::ReleaseVertexArray() const
	 *		- releases vertex array
	 *		- call after drawing
	 */
	void ReleaseVertexArray() const;

	/*
	 *	void CDudeMesh::DrawShadow(Vector2f v_pos, Vector3f v_light_pos) const
	 *		- draws dude shadow at position v_pos
	 *		- extrude shadow as if cast from light v_light_pos
	 *		- note this requires shadow texture to be bound and dude vertex array not to be bound
	 */
	void DrawShadow(Vector2f v_pos, Vector3f v_light_pos) const;

protected:
	bool Create(int n_section_num, int n_subsect_size, int n_side_num, float f_stand_size);
};

#endif //__DUDE_MESH_INCLUDED
