/*
								+----------------------------------+
								|                                  |
								|   ***  Game board classes  ***   |
								|                                  |
								|   Copyright  -tHE SWINe- 2007   |
								|                                  |
								|          GameBoard.cpp           |
								|                                  |
								+----------------------------------+
*/

#include <vector>
#include "../berLame_src/CallStack.h"
#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glut.h>
#include <GL/glext.h>
#include <stdio.h>
#include <math.h>
#include <algorithm>
#include "../berLame_src/Vector.h"
#include "GameBoard.h"

/*
 *								 === TBoardMetrics ===
 */

/*
 *	TBoardMetrics::TBoardMetrics(const TBoardMetrics_In &r_t_bm_in)
 *		- default constructor; converts TBoardMetrics_In to TBoardMetrics
 */
TBoardMetrics::TBoardMetrics(const TBoardMetrics_In &r_t_bm_in)
{
	float f_limit = 1 - 2 * r_t_bm_in.f_board_padding;
	float f_size = (f_limit - r_t_bm_in.f_stand_padding *
		r_t_bm_in.n_stand_num) / r_t_bm_in.n_stand_num;
	float f_min = -f_limit * .5f + .5f * r_t_bm_in.f_stand_padding;

	v_corner_pos = Vector2f(f_min, f_min);
	v_stand_size = Vector2f(f_size, f_size);
	v_stand_offset = Vector2f(f_size + r_t_bm_in.f_stand_padding,
		f_size + r_t_bm_in.f_stand_padding);
	v_first_center_pos = v_corner_pos + Vector2f(f_size * .5f, f_size * .5f);
}

/*
 *								 === ~TBoardMetrics ===
 */

/*
 *								 === TStand ===
 */

/*
 *	TStand::TStand(int _x, int _y, int _n_type = stand_Common)
 *		- default constructor, creates stand of type _n_type at _x, _y
 */
TStand::TStand(int _x, int _y, int _n_type)
	:x(_x), y(_y), n_type(_n_type)
{}

/*
 *	bool TStand::operator <(const TStand &r_t_other) const
 *		- less-than operator for sorting (sorting to minimize texture switches)
 *		- returns true if type of this stand is less than type of r_t_other
 */
bool TStand::operator <(const TStand &r_t_other) const
{
	return n_type < r_t_other.n_type;
}

/*
 *	void TStand::Draw(const TBoardMetrics &r_t_bm) const
 *		- draws stand, r_t_bm is board metrics, defining stand positions and dimensions
 */
void TStand::Draw(const TBoardMetrics &r_t_bm) const
{
	Vector2f v_pos = r_t_bm.v_corner_pos + r_t_bm.v_stand_offset * Vector2f(x, y);
	Vector2f v_end = v_pos + r_t_bm.v_stand_size;

	glBegin(GL_QUADS);
	glNormal3f(0, 1, 0);
	glTexCoord2f(0, 0);
	glVertex3f(v_pos.x, 0, v_pos.y);
	glTexCoord2f(1, 0);
	glVertex3f(v_end.x, 0, v_pos.y);
	glTexCoord2f(1, 1);
	glVertex3f(v_end.x, 0, v_end.y);
	glTexCoord2f(0, 1);
	glVertex3f(v_pos.x, 0, v_end.y);
	glEnd();
}

/*
 *								 === ~TStand ===
 */

/*
 *								 === CBoardGenerator ===
 */

/*
 *	static bool CBoardGenerator::PlaceStands(const TBoardMetrics_In &r_t_bm_in,
 *		std::vector<TStand> &r_stand_list)
 *		- places stands specified by r_t_bm_in to r_stand_list
 *		- returns true on success, false on failure
 */
bool CBoardGenerator::PlaceStands(const TBoardMetrics_In &r_t_bm_in,
	std::vector<TStand> &r_stand_list)
{
	r_stand_list.clear();
	int n_stands_player = r_t_bm_in.n_stand_num - 1 + r_t_bm_in.n_house_size * r_t_bm_in.n_house_size * 2;
	r_stand_list.reserve(4 * n_stands_player);
	if(r_stand_list.capacity() < 4 * n_stands_player)
		return false;

	for(int x = 0; x < r_t_bm_in.n_stand_num - 1; ++ x) {
		if(x < r_t_bm_in.n_house_space)
			r_stand_list.push_back(TStand(x, r_t_bm_in.n_house_space, (x)? stand_Common : stand_Red));
		else if(x < 2 * r_t_bm_in.n_house_space)
			r_stand_list.push_back(TStand(r_t_bm_in.n_house_space, x + 1 - r_t_bm_in.n_house_space));
		else
			r_stand_list.push_back(TStand(x - r_t_bm_in.n_house_space, 0));
	}
	for(int y = 0; y < r_t_bm_in.n_house_size; ++ y) {
		for(int x = 0; x < r_t_bm_in.n_house_size; ++ x)
			r_stand_list.push_back(TStand(x, y, stand_Red));
	}
	for(int i = 0; i < r_t_bm_in.n_house_size * r_t_bm_in.n_house_size; ++ i)
		r_stand_list.push_back(TStand(i + 1, r_t_bm_in.n_house_space + 1, stand_Red));
	// generate one quarter stands

	for(int n_clone = stand_Green; n_clone <= stand_Yellow; ++ n_clone) {
		int n_stands_player = r_t_bm_in.n_stand_num + r_t_bm_in.n_house_size * r_t_bm_in.n_house_size * 2;
		for(int i = r_stand_list.size() - n_stands_player, n = r_stand_list.size(); i < n; ++ i) {
			TStand t_orig = r_stand_list[i];
			r_stand_list.push_back(TStand(t_orig.y, r_t_bm_in.n_stand_num - 1 - t_orig.x,
				(t_orig.n_type != stand_Common)? n_clone : stand_Common));
		}
	}
	// clone-rotate stands

	//std::stable_sort(r_stand_list.begin(), r_stand_list.end());
	// sort stands by type

	return true;
}

/*
 *	static bool CBoardGenerator::PlaceDudes(const TBoardMetrics_In &r_t_bm_in,
 *		std::vector<TDudeInstance> &r_dude_list)
 *		- places dudes specified by r_t_bm_in to r_stand_list
 *		- returns true on success, false on failure
 */
bool CBoardGenerator::PlaceDudes(const TBoardMetrics_In &r_t_bm_in,
	std::vector<TDudeInstance> &r_dude_list)
{
	r_dude_list.reserve(r_t_bm_in.n_house_size * r_t_bm_in.n_house_size * 4);
	if(r_dude_list.capacity() < r_t_bm_in.n_house_size * r_t_bm_in.n_house_size * 4)
		return false;

	TBoardMetrics t_bm(r_t_bm_in);

	int n_name = 0;
	for(int y = 0; y < r_t_bm_in.n_house_size; ++ y) {
		for(int x = 0; x < r_t_bm_in.n_house_size; ++ x) {
			Vector2f v_pos = t_bm.v_first_center_pos + t_bm.v_stand_offset * Vector2f(x, y);
			r_dude_list.push_back(TDudeInstance(v_pos, ++ n_name, stand_Red));
		}
	}
	// place dudes

	int n_dudes_player = r_t_bm_in.n_house_size * r_t_bm_in.n_house_size;
	for(int i = 0; i < 3; ++ i) {
		int n_first = n_dudes_player * i;
		int n_last = n_first + n_dudes_player;
		for(int j = n_first; j < n_last; ++ j) {
			TDudeInstance t_di = r_dude_list[j];
			r_dude_list.push_back(TDudeInstance(Vector2f(t_di.v_pos.y,
				-t_di.v_pos.x), ++ n_name, t_di.n_type + 1));
		}
	}
	// place more dudes

	return true;
}

/*
 *								 === ~CBoardGenerator ===
 */
