/*
								+----------------------------------+
								|                                  |
								|   ***  Game board classes  ***   |
								|                                  |
								|   Copyright  -tHE SWINe- 2007   |
								|                                  |
								|           GameBoard.h            |
								|                                  |
								+----------------------------------+
*/

#ifndef __GAME_BOARD_INCLUDED
#define __GAME_BOARD_INCLUDED

/*
 *	struct TBoardMetrics_In
 *		- game board metrics specified from user input (ie. human-readable form)
 */
struct TBoardMetrics_In {
	int n_house_space;
	int n_house_size;
	int n_stand_num;
	float f_stand_padding;
	float f_board_padding;
};

/*
 *	struct TBoardMetrics
 *		- game board metrics in machine-friendly form
 */
struct TBoardMetrics {
	Vector2f v_corner_pos;
	Vector2f v_first_center_pos;
	Vector2f v_stand_size;
	Vector2f v_stand_offset;

	/*
	 *	TBoardMetrics::TBoardMetrics(const TBoardMetrics_In &r_t_bm_in)
	 *		- default constructor; converts TBoardMetrics_In to TBoardMetrics
	 */
	TBoardMetrics(const TBoardMetrics_In &r_t_bm_in);
};

/*
 *	<unnamed>
 *		- stand type names
 */
enum {
	stand_Common = 0,
	stand_Red,
	stand_Green,
	stand_Blue,
	stand_Yellow
};

/*
 *	struct TStand
 *		- stand is one place where dude stands
 */
struct TStand {
	int x, y;
	int n_type;

	/*
	 *	TStand::TStand(int _x, int _y, int _n_type = stand_Common)
	 *		- default constructor, creates stand of type _n_type at _x, _y
	 */
	TStand(int _x, int _y, int _n_type = stand_Common);

	/*
	 *	bool TStand::operator <(const TStand &r_t_other) const
	 *		- less-than operator for sorting (sorting to minimize texture switches)
	 *		- returns true if type of this stand is less than type of r_t_other
	 */
	bool operator <(const TStand &r_t_other) const;

	/*
	 *	void TStand::Draw(const TBoardMetrics &r_t_bm) const
	 *		- draws stand, r_t_bm is board metrics, defining stand positions and dimensions
	 */
	void Draw(const TBoardMetrics &r_t_bm) const;
};

/*
 *	struct TDudeInstance
 *		- a single "instance" of dude
 */
struct TDudeInstance {
	Vector2f v_pos;
	int n_id, n_type;

	/*
	 *	TDudeInstance(Vector2f _v_pos, int _n_id, int _n_type = stand_Red)
	 *		- default constructor
	 */
	TDudeInstance(Vector2f _v_pos, int _n_id, int _n_type = stand_Red)
		:v_pos(_v_pos), n_id(_n_id), n_type(_n_type)
	{
		_ASSERTE(n_id >= 0 && n_type >= stand_Red && n_type <= stand_Yellow);
	}
};

/*
 *	class CBoardGenerator
 *		- implements function for placing game board stands
 */
class CBoardGenerator {
public:
	/*
	 *	static bool CBoardGenerator::PlaceStands(const TBoardMetrics_In &r_t_bm_in,
	 *		std::vector<TStand> &r_stand_list)
	 *		- places stands specified by r_t_bm_in to r_stand_list
	 *		- returns true on success, false on failure
	 */
	static bool PlaceStands(const TBoardMetrics_In &r_t_bm_in,
		std::vector<TStand> &r_stand_list);

	/*
	 *	static bool CBoardGenerator::PlaceDudes(const TBoardMetrics_In &r_t_bm_in,
	 *		std::vector<TDudeInstance> &r_dude_list)
	 *		- places dudes specified by r_t_bm_in to r_stand_list
	 *		- returns true on success, false on failure
	 */
	static bool PlaceDudes(const TBoardMetrics_In &r_t_bm_in,
		std::vector<TDudeInstance> &r_dude_list);
};

#endif //__GAME_BOARD_INCLUDED
