/*
								+-----------------------------------+
								|                                   |
								|    ***  Clovece nezlob se  ***    |
								|                                   |
								|   Copyright   -tHE SWINe- 2007   |
								|                                   |
								|              Main.cpp             |
								|                                   |
								+-----------------------------------+
*/

#if defined(_MSC_VER) && !defined(__MWERKS__)
#pragma warning(disable:4786)
#endif
// disable warning C4786 "identifier was truncated to '255' characters in the debug information"

#include <vector>
#include "../berLame_src/CallStack.h"
#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glut.h>
#include <GL/glext.h>
#include <stdio.h>
#include <math.h>
#include <algorithm>
#include <utility>
#include "../berLame_src/Vector.h"
#include "../berLame_src/OpenGL20.h"
#include "../berLame_src/OpenGLState.h"
#include "../berLame_src/Texture.h"
#include "ProcTex.h"
#include "Dude.h"
#include "GameBoard.h"

static Vector2f v_mouse;
// mouse cursor in OpenGL coordinates

static float f_scene_z = -1.3f;
static float f_angle_y = 0.03f, f_angle_x = .85f;
// scene rotation

static int n_width = 640, n_height = 480;
static bool b_fullscreen = false;
// window size

static CGLState *p_state;
static CGLTexture_2D *p_board_tex, *p_wood_tex, *p_stand_tex,
	*p_color_stand_texs[4], *p_dude_shadow_tex;
static Vector2f v_dude_shadow_offset;
// OpenGL objects

static CDudeMesh *p_dude;
// dude mesh

static std::vector<TStand> stand_list;
// stand list

static TBoardMetrics_In t_bm_in = {4, 2, 11, .01f, .05f};
static TBoardMetrics t_bm(t_bm_in);
// board metrics

static std::vector<TDudeInstance> dude_list;
// list of dudes

// here we usualy draw glViewport and all commands
// to be called just once before drawing many frames
void onResize(int n_new_width, int n_new_height)
{
	glViewport(0, 0, n_width = n_new_width, n_height = n_new_height);
	// set new viewport
}

void DrawQuad(int n_tess, float f_size = 1, float f_tex_size = 1)
{
	std::vector<std::pair<Vector2f, Vector3f> > vertices;
		
	int tess = n_tess;
	if(!(tess & 1))
		++ tess; // make sure it's odd
	int vertexNum = 2 * (tess + 1) * tess;
	for(int i = 0; i < vertexNum; ++ i) {
		int nx = (i / 2) % (tess + 1);
		int ny = (i / (2 * (tess + 1))); + (1 - (i % 2));
		ny += ((ny + i) % 2);

		float fs = (float)nx / tess; 
		float ft = (float)ny / tess;

		float fx = (fs - .5f) * f_size;
		float fy = ((1 - ft) - .5f) * f_size;

		if((i / (2 * (tess + 1))) % 2 == 1) {
			fx *= -1;
			fs = 1.0f - fs;
		}

		vertices.push_back(std::pair<Vector2f, Vector3f>(Vector2f(fs, ft), Vector3f(fx, 0, fy)));
	}

	glBegin(GL_TRIANGLE_STRIP);
	glNormal3f(0, 1, 0);
	for(int i = 0; i < vertices.size(); ++ i) {
		Vector2f v_tc = vertices[i].first;
		Vector3f v_pos = vertices[i].second;

		glTexCoord2f(v_tc.x, v_tc.y);
		glVertex3f(v_pos.x, v_pos.y, v_pos.z);
	}
	glEnd();


	/*float f_dxy = f_size / n_tess;
	float f_dst = f_tex_size / n_tess;
	// f_dxy je delka hrany jednoho ctverecku v prostoru
	// f_dst je delka hrany jednoho ctverecku v texture

	glNormal3f(0, 1, 0);
	// pro cely ctverec je spolecna jedna normala

	float f_y = -.5f * f_size, f_t = 0;
	// s y-souradnici zaciname nahore, stejne tak odpovidajici t souradnice nahore

	glBegin(GL_TRIANGLE_STRIP);
	for(int y = 0; y < n_tess; ++ y, f_y += f_dxy, f_t += f_dst) {
		float f_y1 = f_y + f_dxy, f_t1 = f_t + f_dst;
		// spocitame y a t souradnici pro dalsi radek

		if(y & 1) {
			float f_x = .5f * f_size, f_s = 1;
			// s x-souradnici zaciname vpravo, stejne tak s vpravo

			for(int x = 0; x <= n_tess; ++ x, f_x -= f_dxy, f_s -= f_dst) {
				glTexCoord2f(f_s, f_t1);
				glVertex3f(f_x, 0, f_y1);
				glTexCoord2f(f_s, f_t);
				glVertex3f(f_x, 0, f_y);
				if(x == n_tess) {
					glVertex3f(f_x, 0, f_y);
					// pokud to je posledni tak vertex posleme jeste jednou. na texcoord
					// nezalezi, protoze generovany trojuhelnik bude mit nulovou sirku -> nebude videt
				}
			}
		} else {
			float f_x = -.5f * f_size, f_s = 0;
			// s x-souradnici zaciname vlevo, stejne tak s vlevo

			for(int x = 0; x <= n_tess; ++ x, f_x += f_dxy, f_s += f_dst) {
				glTexCoord2f(f_s, f_t1);
				glVertex3f(f_x, 0, f_y1);
				glTexCoord2f(f_s, f_t);
				glVertex3f(f_x, 0, f_y);
				if(x == n_tess) {
					glVertex3f(f_x, 0, f_y);
					// pokud to je posledni tak vertex posleme jeste jednou. na texcoord
					// nezalezi, protoze generovany trojuhelnik bude mit nulovou sirku -> nebude videt
				}
			}
		}
	}
	glEnd();*/
}

// here we render frame
void onDisplay(CGLState *p_state, bool b_pick = false, int x = 0, int y = 0)
{
	p_state->ClearColor4f(0, 0, 0, 1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	if(b_pick) {
		GLint p_viewport[4];
		glGetIntegerv(GL_VIEWPORT, p_viewport);
		gluPickMatrix(x, y, 5, 5, p_viewport);
	}

	gluPerspective(45, (float)n_width / n_height, .01f, 10);
	// set perspective projection

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	// reset matrix

	glTranslatef(0, .1f, f_scene_z);
	glRotatef(f_angle_x * 90, 1, 0, 0);
	glRotatef(f_angle_y * 90, 0, 1, 0);
	// rotate/translate scene mouse-wisely

	p_state->DisableDepthTest();
	p_state->EnableCullFace();
	p_state->FrontFace(GL_CW);
	p_state->DisableTexture2D();
	p_state->DisableBlend();
	p_state->EnableLighting();
	p_state->Color3f(1, 1, 1);

	p_state->EnableLight(0);
	p_state->LightAmbient(0, &Vector4f(1, 1, 1, 1)[0]);
	p_state->LightDiffuse(0, &Vector4f(1, 1, 1, 1)[0]);
	p_state->LightSpecular(0, &Vector4f(.8f, .8f, .8f, 1)[0]);
	const Vector4f v_light_pos(-1, 3, -1, 1);
	p_state->LightPosition(0, &v_light_pos.x);
	p_state->LightSpotDirection(0, &Vector3f(1, -3, 1)[0]);
	p_state->LightSpotCutoff(0, 20);
	p_state->LightSpotExponent(0, 50);
	// set light props

	p_state->MaterialAmbient(&Vector4f(.2f, .2f, .2f, 1)[0]);
	p_state->MaterialDiffuse(&Vector4f(1, 1, 1, 1)[0]);
	p_state->MaterialSpecular(&Vector4f(1, 1, 1, 1)[0]);
	p_state->MaterialShininess(128);
	// set wood material props

	p_wood_tex->Bind_Enable(p_state);
	DrawQuad(20, 4, 1);
	// draw ground plane

	p_state->MaterialSpecular(&Vector4f(.2f, .2f, .2f, 1)[0]);
	p_state->MaterialShininess(50);
	// set paper material props

	p_board_tex->Bind_Enable(p_state);
	DrawQuad(10, 1, 1);
	// draw ground plane

	p_state->EnableBlend();
	p_state->BlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	// figure stands are blended-in

	for(int i = 0; i < stand_list.size(); ++ i) {
		if(stand_list[i].n_type != stand_Common)
			p_color_stand_texs[stand_list[i].n_type - 1]->Bind_Enable(p_state);
		else
			p_stand_tex->Bind_Enable(p_state);
		stand_list[i].Draw(t_bm);
	}
	// draw figure stands

	p_state->BlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_COLOR);
	p_state->EnableAlphaTest();
	p_state->AlphaFunc(GL_GREATER, .1f);
	p_state->DisableLighting();
	glColor3f(1, 1, 1);
	p_dude_shadow_tex->Bind_Enable(p_state);
	// shadows are blended-in

	for(int i = 0, n = dude_list.size(); i < n; ++ i)
		p_dude->DrawShadow(dude_list[i].v_pos, v_light_pos.v_xyz());
	// draw dude shadows

	p_state->DisableAlphaTest();
	p_state->DisableTexture2D();
	p_state->DisableBlend();
	p_state->EnableLighting();
	p_state->EnableDepthTest();
	// re-enable lighting for dudes

	p_state->MaterialSpecular(&Vector4f(1, 1, 1, 1)[0]);
	p_state->MaterialShininess(80);
	// set generic dude mtl props

	p_dude->BindVertexArray();
	for(int i = 0, n = dude_list.size(); i < n; ++ i) {
		switch(dude_list[i].n_type) {
		case stand_Red:
			p_state->MaterialAmbient(&Vector4f(.5f, .2f, .2f, 1)[0]);
			p_state->MaterialDiffuse(&Vector4f(1, .2f, .2f, 1)[0]);
			// red dudes
			break;
		case stand_Green:
			p_state->MaterialAmbient(&Vector4f(.2f, .5f, .2f, 1)[0]);
			p_state->MaterialDiffuse(&Vector4f(.2f, 1, .2f, 1)[0]);
			// green dudes
			break;
		case stand_Blue:
			p_state->MaterialAmbient(&Vector4f(.4f, .4f, .7f, 1)[0]);
			p_state->MaterialDiffuse(&Vector4f(.2f, .2f, 1, 1)[0]);
			// blue dudes
			break;
		case stand_Yellow:
			p_state->MaterialAmbient(&Vector4f(.5f, .5f, .1f, 1)[0]);
			p_state->MaterialDiffuse(&Vector4f(1, 1, .1f, 1)[0]);
			// yellow dudes
			break;
		}
		// set proper material

		if(b_pick)
			glPushName(dude_list[i].n_id);
		p_dude->Draw(dude_list[i].v_pos);
		if(b_pick)
			glPopName();
	}
	p_dude->ReleaseVertexArray();
	// draw dudes

	if(glGetError() != GL_NO_ERROR)
		fprintf(stderr, "error: OpenGL error(s) occured\n");
}

void PrepareShadowTexture(CGLState *p_state)
{
	glViewport(0, 0, p_dude_shadow_tex->n_Width(), p_dude_shadow_tex->n_Height());
	p_state->ClearColor4f(0, 0, 0, 0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90, (float)64 / 128, .01f, 10);
	// set perspective projection

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	// reset matrix

	float f_dude_width = 1.5f * 2 * p_dude->f_Scale();
	float f_dude_height = 7 * p_dude->f_Scale();
	// dude size

	glTranslatef(0, 0, -1);
	f_dude_width = .5f / f_dude_width;
	glScalef(f_dude_width, f_dude_width, f_dude_width);
	glTranslatef(0, -f_dude_height / 2, 0);
	// position dude so it should fit into the texture

	Matrix4f t_projection;
	glGetFloatv(GL_PROJECTION_MATRIX, &t_projection[0][0]);
	Matrix4f t_modelview;
	glGetFloatv(GL_MODELVIEW_MATRIX, &t_modelview[0][0]);
	// get matrices

	t_modelview = t_projection * t_modelview;
	// multiply em

	Vector4f v_pos3d = t_modelview * Vector4f(0, 0, 0, 1);
	v_dude_shadow_offset = v_pos3d.v_xy() /= v_pos3d.w;
	// 2D position of dude base in texture coordinates

	p_dude->PrepareShadow(v_dude_shadow_offset);

	p_state->DisableTexture2D();
	p_state->EnableDepthTest();
	p_state->Color3f(.25f, .25f, .25f);
	p_dude->BindVertexArray();
	p_dude->Draw();
	p_dude->ReleaseVertexArray();
	// draw dude

	p_dude_shadow_tex->Bind_Enable(p_state);
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0,
		p_dude_shadow_tex->n_Width(), p_dude_shadow_tex->n_Height());
	// copy result to texture
}

// onDisplay function adapter, which moves p_state to stack
void onDisplayAdapter()
{
	if(!p_state)
		return;

	static bool b_shadow_ready = false;
	if(!b_shadow_ready) {
		//PrepareShadowTexture(p_state);
		b_shadow_ready = true;

		glViewport(0, 0, n_width, n_height);
		// put back correct viewport
	}

	onDisplay(p_state);

	glutSwapBuffers();
	// display results
}

int n_hold_dude = -1;

void onSelect(int x, int y)
{
	if(!p_state)
		return;

	GLuint p_buffer[256];
	const int n_buffer_size = sizeof(p_buffer) / sizeof(p_buffer[0]);
	glSelectBuffer(n_buffer_size, p_buffer);
	glRenderMode(GL_SELECT);
	glInitNames();
	// prepare for rendering in selection mode

	onDisplay(p_state, true, x, y);
	// render to small viewport, supply names of objects

	int n_hit_num = glRenderMode(GL_RENDER);
	glViewport(0, 0, n_width, n_height);
	// put it back

	if(n_hit_num) {
		GLuint *p_names = 0;
		int n_close_name_num = 0;

		GLuint *p_ptr = (GLuint*)p_buffer;
		GLuint n_min_z = 0xffffffff;
		for(int i = 0; i < n_hit_num; ++ i) {
			int n_name_num = *p_ptr;
			++ p_ptr;
			if(*p_ptr < n_min_z) {
				n_close_name_num = n_name_num;
				n_min_z = *p_ptr;
				p_names = p_ptr + 2;
			}
			p_ptr += n_name_num + 2;
		}

		if(p_names && n_close_name_num) {
			n_hold_dude = -1;
			for(int i = 0, n = dude_list.size(); i < n; ++ i) {
				if(dude_list[i].n_id == *p_names) {
					n_hold_dude = i;
					break;
				}
			}
		}
		// pick something closest from hit buffer
	}
}

static bool b_left_button = false;

// here we receive notifications on mouse clicks
// x and y are cursor coordinates in pixels
void onMouseClick(int button, int state, int x, int y)
{
	v_mouse.x = (2.0f * x) / n_width - 1;
	v_mouse.y = 1 - (2.0f * y) / n_height;
	// get current mouse position

	if(n_hold_dude >= 0 && n_hold_dude < dude_list.size()) {
		float f_best_dist = t_bm.v_stand_size.x * .5f;
		int n_best_stand = -1;
		Vector2f v_best_pos;

		Vector2f v_dude_pos = dude_list[n_hold_dude].v_pos;
		for(int i = 0, n = stand_list.size(); i < n; ++ i) {
			Vector2f v_pos(t_bm.v_first_center_pos + t_bm.v_stand_offset *
				Vector2f(stand_list[i].x, stand_list[i].y));
			float f_dist = (v_pos - v_dude_pos).f_Length();
			if(f_dist < f_best_dist) {
				f_best_dist = f_dist;
				n_best_stand = i;
				v_best_pos = v_pos;
			}
		}

		if(n_best_stand >= 0)
			dude_list[n_hold_dude].v_pos = v_best_pos;
	}
	// in case we were holding some dude, snap it to close stand if any

	n_hold_dude = -1;

	switch(button) {
    case GLUT_LEFT_BUTTON:
		b_left_button = true;
		if(!state)
			onSelect(x, n_height - 1 - y);
		break;
    case GLUT_MIDDLE_BUTTON:
    case GLUT_RIGHT_BUTTON:
		b_left_button = false;
		break;
	};
	// in case button is going to be important for dragging, it's necessary to remember it

	glutPostRedisplay();
}

// here we receive notifications on mouse motion when dragging
// something, x and y are cursor coordinates in pixels
void onMouseMotion(int x, int y)
{
	float f_new_mouse_x = (2.0f * x) / n_width - 1;
	float f_new_mouse_y = 1 - (2.0f * y) / n_height;
	// get current mouse position

	if(!b_left_button) {
		f_angle_y += f_new_mouse_x - v_mouse.x;
		f_angle_x += f_new_mouse_y - v_mouse.y;

		if(f_angle_x < .2f)
			f_angle_x = .2f;
		else if(f_angle_x > 1)
			f_angle_x = 1;
	} /*else
		f_scene_z += f_new_mouse_x - v_mouse.x;*/
	// rotate scene

	if(n_hold_dude >= 0 && n_hold_dude < dude_list.size()) {
		Matrix4f t_projection, t_modelview;
		glGetFloatv(GL_PROJECTION_MATRIX, &t_projection[0][0]);
		glGetFloatv(GL_MODELVIEW_MATRIX, &t_modelview[0][0]);
		// get matrices

		t_modelview = t_projection * t_modelview;
		t_modelview.FullInvert();
		// multiply and invert em

		for(int n_pass = 0;; ++ n_pass) {
			Vector4f v_far4 = t_modelview * Vector4f(v_mouse.x, v_mouse.y, 2 * 1 - 1, 1);
			Vector4f v_near4 = t_modelview * Vector4f(v_mouse.x, v_mouse.y, 2 * 0 - 1, 1);
			// mouse coords already are normalized to [-1, 1], do it for z

			Vector3f v_far = v_far4.v_xyz() / v_far4.w;
			Vector3f v_near = v_near4.v_xyz() / v_near4.w;
			// mouse coords to worldspace

			Vector3f v_ray(v_far - v_near);
			Vector3f v_intersection = v_near - v_ray * (v_near.y / v_ray.y);
			// intersection with ground plane

			if(n_pass) {
				dude_list[n_hold_dude].v_pos += v_intersection.v_xz();
				break;
			} else {
				v_mouse.x = f_new_mouse_x;
				v_mouse.y = f_new_mouse_y;
				dude_list[n_hold_dude].v_pos -= v_intersection.v_xz();
			}
		}
		// very intricate size optimization; it calculates distance between old
		// projected mouse coords and new ones and adds it to dude position
	}
	// move dude

	v_mouse.x = f_new_mouse_x;
	v_mouse.y = f_new_mouse_y;
	// update mouse coords

	glutPostRedisplay();
	// asume we changed something and now we want to redraw ...
}

bool b_HaveExtension(const char *p_s_ext_name)
{
	const char *p_s_exts = (const char*)glGetString(GL_EXTENSIONS);
	const char *p_s_ptr;
	if(p_s_exts && (p_s_ptr = strstr(p_s_exts, p_s_ext_name))) {
		unsigned int n_length = strlen(p_s_ext_name);
		if((p_s_ptr == p_s_exts || isspace(p_s_ptr[-1])) &&
		   (p_s_ptr[n_length] == 0 || isspace(p_s_ptr[n_length])))
			return true;
		// in case before is whitespace or beginning and after is whitespace or end, we have it
	}
	// determine wheter extension is supported

	return false;
}

Vector4f v_StandPaperFunc(Vector2f v_pos)
{
	Vector4f v_tone;
	CProcTexGen::WebColor(0xfefae4, v_tone); // paper
	return CProcTexGen::v_StandPaperFunc(v_pos, v_tone);
}

Vector4f v_StandPaperRFunc(Vector2f v_pos)
{
	Vector4f v_tone;
	CProcTexGen::WebColor(0xff0000, v_tone); // red
	return CProcTexGen::v_StandPaperFunc(v_pos, v_tone);
}

Vector4f v_StandPaperGFunc(Vector2f v_pos)
{
	Vector4f v_tone;
	CProcTexGen::WebColor(0x00ff00, v_tone); // green
	return CProcTexGen::v_StandPaperFunc(v_pos, v_tone);
}

Vector4f v_StandPaperBFunc(Vector2f v_pos)
{
	Vector4f v_tone;
	CProcTexGen::WebColor(0x0000ff, v_tone); // blue
	return CProcTexGen::v_StandPaperFunc(v_pos, v_tone);
}

Vector4f v_StandPaperYFunc(Vector2f v_pos)
{
	Vector4f v_tone;
	CProcTexGen::WebColor(0xffff00, v_tone); // yellow
	return CProcTexGen::v_StandPaperFunc(v_pos, v_tone);
}

void Cleanup()
{
	if(p_board_tex)
		delete p_board_tex;
	if(p_wood_tex)
		delete p_wood_tex;
	if(p_stand_tex)
		delete p_stand_tex;
	for(int n = 0; n < 4; ++ n) {
		if(p_color_stand_texs[n])
			delete p_color_stand_texs[n];
	}
	if(p_dude_shadow_tex)
		delete p_dude_shadow_tex;
	if(p_dude)
		delete p_dude;
	if(p_state)
		delete p_state;
}

void onKeyPressed(unsigned char n_key_code, int n_x, int n_y)
{
	if(n_key_code == 27) {
		Cleanup();
		exit(0);
	} else if(n_key_code == '+' || n_key_code == 'a') {
		f_scene_z += .03f;
		if(f_scene_z > -.1f)
			f_scene_z = -.1f;
		glutPostRedisplay();
	} else if(n_key_code == '-' || n_key_code == 'z' || n_key_code == 'y') {
		f_scene_z -= .03f;
		glutPostRedisplay();
	} else if(n_key_code == 'f') {
		if(!b_fullscreen) {
			glutFullScreen();
			b_fullscreen = true;
		} else {
			glutReshapeWindow(640, 480);
			b_fullscreen = false;
		}
	}
}

bool Init()
{
	p_board_tex = 0;
	p_wood_tex = 0;
	p_stand_tex = 0;
	for(int n = 0; n < 4; ++ n)
		p_color_stand_texs[n] = 0;
	p_dude_shadow_tex = 0;
	p_dude = 0;
	p_state = 0;
	// clear pointers

	if(!(p_state = new CGLState())) {
		fprintf(stderr, "error: not enough memory\n");
		return false;
	}
	// create state guard

	printf("generating dude mesh ...\n");
	if(!(p_dude = new CDudeMesh(t_bm.v_stand_size.x, 5, 12))) {
		fprintf(stderr, "error: not enough memory\n");
		return false;
	}
	// build dude mesh

	printf("generating textures ...\n");
	printf("\tboard paper\n");
	if(!(p_board_tex = CProcTexGen::p_BuildTexture2D(p_state, 256, 256,
	   GL_RGB, CProcTexGen::v_BoardPaperFunc)))
		return false;
	printf("\twood plank\n");
	if(!(p_wood_tex = CProcTexGen::p_BuildTexture2D(p_state, 512, 512,
	   GL_RGB, CProcTexGen::v_WoodPlankFunc)))
		return false;
	printf("\tstand circles\n");
	if(!(p_stand_tex = CProcTexGen::p_BuildTexture2D(p_state, 32, 32,
	   GL_RGBA, v_StandPaperFunc)))
		return false;
	if(!(p_color_stand_texs[0] = CProcTexGen::p_BuildTexture2D(p_state, 32, 32,
	   GL_RGBA, v_StandPaperRFunc)))
		return false;
	if(!(p_color_stand_texs[1] = CProcTexGen::p_BuildTexture2D(p_state, 32, 32,
	   GL_RGBA, v_StandPaperGFunc)))
		return false;
	if(!(p_color_stand_texs[2] = CProcTexGen::p_BuildTexture2D(p_state, 32, 32,
	   GL_RGBA, v_StandPaperBFunc)))
		return false;
	if(!(p_color_stand_texs[3] = CProcTexGen::p_BuildTexture2D(p_state, 32, 32,
	   GL_RGBA, v_StandPaperYFunc)))
		return false;
	if(!(p_dude_shadow_tex = CProcTexGen::p_BuildTexture2D(p_state, 128, 256, GL_RGBA, 0)))
		return false;
	PrepareShadowTexture(p_state);
	// generate textures

	if(!CBoardGenerator::PlaceStands(t_bm_in, stand_list) ||
	   !CBoardGenerator::PlaceDudes(t_bm_in, dude_list))
		return false;
	// place figure stands and figures

	return true;
}

int main(int n_arg_num, const char **p_arg_list)
{
	for(int i = 1; i < n_arg_num; ++ i) {
		if(!strcmp(p_arg_list[i], "--house-size")) {
			if(i + 1 == n_arg_num) {
				fprintf(stderr, "error: argument \'%s\'"
					" must be followed by value\n", p_arg_list[i]);
				break;
			}
			t_bm_in.n_house_size = atol(p_arg_list[++ i]);
		} else if(!strcmp(p_arg_list[i], "--house-space")) {
			if(i + 1 == n_arg_num) {
				fprintf(stderr, "error: argument \'%s\'"
					" must be followed by value\n", p_arg_list[i]);
				break;
			}
			t_bm_in.n_house_space = atol(p_arg_list[++ i]);
		} else if(!strcmp(p_arg_list[i], "--board-size")) {
			if(i + 1 == n_arg_num) {
				fprintf(stderr, "error: argument \'%s\'"
					" must be followed by value\n", p_arg_list[i]);
				break;
			}
			t_bm_in.n_stand_num = atol(p_arg_list[++ i]);
		} else
			fprintf(stderr, "error: unknown argument \'%s\'\n", p_arg_list[i]);
	}
	t_bm = TBoardMetrics(t_bm_in);
	// parse cmdline if any

	glutInit(&n_arg_num, (char **)p_arg_list);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutInitWindowSize(n_width, n_height);
	glutCreateWindow("Game by -tHE SWINe-");
	// init OpenGL using GLUT

	/*printf("\t\t\t\t=== Napoveda ===\n\n");

	printf("pravym tlacitkem se da otacet scenou\n");
	printf("levym tlacitkem se da vybirat figurka a jeji umisteni\n");
	printf("tlacitky \'+\' (\'a\') a \'-\' (\'z\', \'y\') lze priblizovat nebo oddalovat scena\n");
	printf("tlacitkem \'f\' se prepne mezi zobrazenim na celou obrazovku nebo do okna\n");
	printf("tlacitkem \'esc\' se program ukonci\n\n");*/

	printf("\t\t\t\t=== Controls ===\n\n");

	printf("drag your right mouse button to rotate the scene\n");
	printf("drag your left mouse button to move arround figures\n");
	printf("use buttons \'+\' (\'a\') and \'-\' (\'z\', \'y\') to zoom the scene\n");
	printf("button \'f\' toggles fullscreen display\n");
	printf("press \'esc\' to exit the game\n\n");

	if(!Init()) {
		Cleanup();
		return -1;
	}
	// initialize app

	if(CGLExtensionHandler::b_SupportedExtension("GL_EXT_separate_specular_color") ||
	   CGLExtensionHandler::b_Support_OpenGL(1, 2))
		p_state->LightModelColorControl(GL_SEPARATE_SPECULAR_COLOR_EXT);
	else
		fprintf(stderr, "warning: GL_EXT_separate_specular_color not supported\n");
	// enable separate specular color

	if(CGLExtensionHandler::b_SupportedExtension("GL_EXT_texture_filter_anisotropic")) {
		float f_aniso;
		glGetFloatv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &f_aniso);
		// anisotropy value

		p_stand_tex->r_Parameters(p_state).Set_Texture_Anisotropy(f_aniso);
		p_board_tex->r_Parameters(p_state).Set_Texture_Anisotropy(f_aniso);
		p_wood_tex->r_Parameters(p_state).Set_Texture_Anisotropy(f_aniso);
		for(int i = 0; i < 4; ++ i)
			p_color_stand_texs[i]->r_Parameters(p_state).Set_Texture_Anisotropy(f_aniso);
		// set anisotropic filtering
	} else
		fprintf(stderr, "warning: GL_EXT_texture_filter_anisotropic not supported\n");
	// enable anisotropic filtering

	glutDisplayFunc(onDisplayAdapter);
	glutReshapeFunc(onResize);
	glutMouseFunc(onMouseClick);
	glutMotionFunc(onMouseMotion);
	glutKeyboardFunc(onKeyPressed);
	// register our callbacks

	glutMainLoop();
	// begin rendering

	return 0;
}
