/*
								+----------------------------------+
								|                                  |
								| ***  Procedural texture gen  *** |
								|                                  |
								|   Copyright  -tHE SWINe- 2007   |
								|                                  |
								|            ProcTex.cpp           |
								|                                  |
								+----------------------------------+
*/

#include <vector>
#include "../berLame_src/CallStack.h"
#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glext.h>
#include <math.h>
#include "../berLame_src/Vector.h"
#include "../berLame_src/OpenGL20.h"
#include "../berLame_src/OpenGLState.h"
#include "../berLame_src/Texture.h"
#include "Noise.h"
#include "ProcTex.h"

/*
 *								=== CProcTexGen ===
 */

/*
 *	static CGLTexture_2D *CProcTexGen::p_BuildTexture2D(CGLState *p_state, int n_width,
 *		int n_height, GLenum n_internal_format, Vector4f (*p_tex_func)(Vector2f v_pos))
 *		- builds procedural texture n_width per n_height pixels of format n_internal_format
 *		- pixel values are generated by p_tex_func function which is passed relative texcoords
 *		  (in range [0, 1] or more precisely [1 / pixel-size, 1 - 1 / pixel-size])
 *		- note p_tex_func can be 0 to create uninitialized texture
 *		- p_state is OpenGL state guard
 *		- returns a new texture object on success, 0 on failure
 */
CGLTexture_2D *CProcTexGen::p_BuildTexture2D(CGLState *p_state, int n_width, int n_height,
	GLenum n_internal_format, Vector4f (*p_tex_func)(Vector2f v_pos))
{
	/*int n_max_tex_size;
	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &n_max_tex_size);
	if(n_width > n_max_tex_size || n_height > n_max_tex_size) {
		fprintf(stderr, "error: max texture size is only %d, requested size is %d x %d\n",
			n_max_tex_size, n_width, n_height);
		if(n_width > n_max_tex_size)
			n_width = n_max_tex_size;
		if(n_height > n_max_tex_size)
			n_height = n_max_tex_size;
	}*/
	// handle max texture size

	unsigned int *p_img_buffer = 0;
	if(p_tex_func) {
		if(!(p_img_buffer = new unsigned int[n_width * n_height]))
			return 0;
		// alloc image buffer

		Vector2f v_step(1.0f / n_width, 1.0f / n_height);
		Vector2f v_pos(.5f * v_step.x, .5f * v_step.y);
		unsigned int *p_img_buffer_ptr = p_img_buffer;
		for(int y = 0; y < n_height; ++ y, v_pos.y += v_step.y) {
			v_pos.x = .5f * v_step.x;
			for(int x = 0; x < n_width; ++ x, v_pos.x += v_step.x, ++ p_img_buffer_ptr) {
				Vector4f v_color(p_tex_func(v_pos));
				// generate color

				for(int n = 0; n < 4; ++ n)
					v_color[n] = (v_color[n] >= 0)? ((v_color[n] <= 1)? v_color[n] : 1) : 0;
				// clamp

				*p_img_buffer_ptr = (unsigned int)(v_color.x * 255) |
					((unsigned int)(v_color.y * 255) << 8) |
					((unsigned int)(v_color.z * 255) << 16) |
					((unsigned int)(v_color.w * 255) << 24);
				// convert to FP
			}
		}
	}
	// generate procedural texture

	bool b_mipmap = CGLExtensionHandler::b_Support_OpenGL(1, 4) ||
		CGLExtensionHandler::b_SupportedExtension("GL_SGIS_generate_mipmap");
	CGLTexture_2D *p_texture;
	if(!(p_texture = new CGLTexture_2D(p_state, n_width, n_height, n_internal_format,
	   b_mipmap, 0, GL_RGBA, GL_UNSIGNED_BYTE, p_img_buffer))) {
		delete[] p_img_buffer;
		return 0;
	}
	if(p_img_buffer)
		delete[] p_img_buffer;
	if(!p_texture->b_Status()) {
		delete p_texture;
		return 0;
	}
	// create texture

	return p_texture;
}

static inline float __f_Turbulence(Vector2f v_pos,
	int n_octave_num, float f_durability, float f_z)
{
	float f_magnitude = 0;
	float f_weight = 1;
	float f_noise = 0;
	for(int i = 0; i < n_octave_num; ++ i, v_pos *= 2, f_weight *= f_durability) {
		f_noise += f_weight * CPerlinNoise::f_Noise(v_pos.x, v_pos.y, 0);
		f_magnitude += f_weight;
	}
	// integrate noise

	return f_noise / f_magnitude;
}

/*
 *	static float CProcTexGen::f_Turbulence(Vector2f v_pos, float f_scale,
 *		int n_octave_num, float f_durability)
 *		- returns turbulence noise at 2D position v_pos with scale f_scale
 *		  (spatial multiplier), n_octave_num octaves and f_durability durability
 *		- note noise is normalized so it gives the same output magnitude at different
 *		  durabilities and octave numbers
 */
float CProcTexGen::f_Turbulence(Vector2f v_pos, float f_scale,
	int n_octave_num, float f_durability)
{
	/*float f_magnitude = 0;
	float f_weight = 1;
	float f_noise = 0;
	for(int i = 0; i < n_octave_num; ++ i, v_pos *= 2, f_weight *= f_durability) {
		f_noise += f_weight * CPerlinNoise::f_Noise(v_pos.x, v_pos.y, 0);
		f_magnitude += f_weight;
	}
	// integrate noise

	return f_noise / f_magnitude;*/

	v_pos *= f_scale;
	return __f_Turbulence(v_pos, n_octave_num, f_durability, 0);
}

/*
 *	static Vector3f CProcTexGen::v_Turbulence(Vector2f v_pos, float f_scale,
 *		int n_octave_num, float f_durability)
 *		- returns turbulence noise vector (three different unrelated noise functions)
 *		  at 2D position v_pos with scale f_scale (spatial multiplier), n_octave_num
 *		  octaves and f_durability durability
 *		- note noise is normalized so it gives the same output magnitude at different
 *		  durabilities and octave numbers
 */
Vector3f CProcTexGen::v_Turbulence(Vector2f v_pos, float f_scale,
	int n_octave_num, float f_durability)
{
	/*v_pos *= f_scale;
	Vector3f v_noise(0, 0, 0);
	float f_magnitude = 0;
	float f_weight = 1;
	for(int i = 0; i < n_octave_num; ++ i, v_pos *= 2, f_weight *= f_durability) {
		v_noise.x += f_weight * CPerlinNoise::f_Noise(v_pos.x, v_pos.y, 0);
		v_noise.y += f_weight * CPerlinNoise::f_Noise(v_pos.x, v_pos.y, 1);
		v_noise.z += f_weight * CPerlinNoise::f_Noise(v_pos.x, v_pos.y, 2);
		f_magnitude += f_weight;
	}
	v_noise /= f_magnitude;*/
	// integrate noise

	v_pos *= f_scale;
	Vector3f v;
	for(int i = 0; i < 3; ++ i)
		v[i] = __f_Turbulence(v_pos, n_octave_num, f_durability, i);
	return v;
}

/*
 *	static Vector4f CProcTexGen::v_BoardPaperFunc(Vector2f v_pos)
 *		- this can be used as texture generation function
 *		- creates paper-like texture with thin red frame
 */
Vector4f CProcTexGen::v_BoardPaperFunc(Vector2f v_pos)
{
	Vector4f v_tone(1, 1, 1, 1);
	float f_quad_x = fabs(v_pos.x - .5f);
	float f_quad_y = fabs(v_pos.y - .5f);
	if((f_quad_x > .46f && f_quad_x < .47f && f_quad_y < .47f) ||
	   (f_quad_y > .46f && f_quad_y < .47f && f_quad_x < .47f))
		WebColor(0xff7c12, v_tone); // red border
	else
		WebColor(0xfefae4, v_tone); // paper
	// decide color tone

	return v_tone * (.8f + .1f * (1 + f_Turbulence(v_pos, 1000, 3, .7f)));
	// final color sum
}

/*
 *	static Vector4f CProcTexGen::v_StandPaperFunc(Vector2f v_pos, Vector4f v_paper_color)
 *		- this can be used as texture generation function, except for v_paper_color
 *		  parameter (requires adaptor function)
 *		- creates paper-like texture with thin red circle and v_paper_color dot inside the
 *		  circle
 *		- note for correct antialiassing this requires texture to have size 32 per 23 pixels
 */
Vector4f CProcTexGen::v_StandPaperFunc(Vector2f v_pos, Vector4f v_paper_color)
{
	Vector4f v_tone(1, 1, 1, 1);

	v_pos -= Vector2f(.5f, .5f);
	float f = v_pos.f_Length();
	const float f_band = 0.12f;
	if(f > .47f - f_band && f < .47f) {
		Vector4f v_red, v_paper;
		WebColor(0xff7c12, v_red);
		WebColor(0xfefae4, v_paper);
		float f_alpha = ((f > .47f - f_band * .5f)? (.47f - f) : f - .47f + f_band) * 32;
		if(f_alpha > 1)
			f_alpha = 1;
		else if(f_alpha < 0)
			f_alpha = 0;
		v_tone = v_paper + (v_red - v_paper) * f_alpha;
	} else if(f < .47f - 1.5f * f_band) {
		Vector4f v_paper;
		WebColor(0xfefae4, v_paper);
		float f_alpha = (.47f - 1.5f * f_band - f) * 32;
		if(f_alpha > 1)
			f_alpha = 1;
		else if(f_alpha < 0)
			f_alpha = 0;
		v_tone = v_paper + (v_paper_color - v_paper) * f_alpha;
	} else
		WebColor(0xfefae4, v_tone); // paper
	// decide color tone

	if(f > .47f)
		v_tone.w = 0;
	// no alpha

	return v_tone * (.8f + .1f * (1 + f_Turbulence(v_pos, 125, 3, .7f)));
	// final color sum
}

/*
 *	static Vector4f CProcTexGen::v_WoodPlankFunc(Vector2f v_pos)
 *		- this can be used as texture generation function
 *		- creates wood plank texture (varnished pine wood taken as the paradigm)
 */
Vector4f CProcTexGen::v_WoodPlankFunc(Vector2f v_pos)
{
	v_pos *= 4;
	// scale the whole thing

	Vector3f v_pos3(v_pos.x + 5, -1, v_pos.y + 2);
	v_pos3 += v_Turbulence(v_pos, 1, 4, .2f);
	// perturbate position

	Vector3f v_axis(1, 3, 20);
	v_axis.Normalize();
	float f_axis_distance = (v_axis * v_axis.f_Dot(v_pos3) - v_pos3).f_Length();
	float f_tone = (sin(50 * f_axis_distance) + 1) * .5f;
	// simple circles arround the axis

	f_tone *= f_tone;
	f_tone *= f_tone;
	// make the transition a bit sharper

	f_tone += f_Turbulence(v_pos, 10, 5, .9f) * .9f;
	f_tone = (f_tone > 1)? 1 : ((f_tone < 0)? 0 : f_tone);
	// affect tone by noise

	Vector4f v_wood0, v_wood1;
	WebColor(0xc34912, v_wood1);
	WebColor(0xf2aa44, v_wood0);
	// two colors of wood

	return v_wood0 + (v_wood1 - v_wood0) * f_tone;
}

/*
 *								=== ~CProcTexGen ===
 */
